#include <stdlib.h>
#include "PerlDocumentCallbackHandler.hpp"

PerlDocumentCallbackHandler::PerlDocumentCallbackHandler() {
    callbackObj = NULL;
}

PerlDocumentCallbackHandler::~PerlDocumentCallbackHandler() {
    if (callbackObj) {
	SvREFCNT_dec(callbackObj); 
	callbackObj = NULL;
    }
}

void
PerlDocumentCallbackHandler::set_callback_obj(SV* object) {
    SvREFCNT_inc(object);
    callbackObj = object;
}

void
PerlDocumentCallbackHandler::startElement(const XMLCh* const name, 
				  AttributeList& attributes) {
    if (!callbackObj) return;

    dSP;

    ENTER;
    SAVETMPS;

    PUSHMARK(SP);
	// first put the callback object on the stack
    XPUSHs(callbackObj);

        // the next argument is the element name
    char *cptr = XMLString::transcode(name);
    SV *string = sv_newmortal();
    sv_setpv(string, (char *)cptr);
    XPUSHs(string);

        // next is the attribute list
    char *class_name = "XML::Xerces::AttributeList";
    HV *hash = newHV();
    HV *stash = gv_stashpv(class_name, FALSE);
    hv_magic(hash, 
    	 (GV *)sv_setref_pv(sv_newmortal(), 
    			    class_name, (void *)&attributes), 
    	 'P');
    XPUSHs(sv_bless(newRV_noinc((SV *)hash), stash));

    PUTBACK;

    perl_call_method("start_element", G_VOID);

    FREETMPS;
    LEAVE;
}

void
PerlDocumentCallbackHandler::endElement(const XMLCh* const name)
{
    if (!callbackObj) return;

    dSP;

    ENTER;
    SAVETMPS;

    PUSHMARK(SP);
	// first put the callback object on the stack
    XPUSHs(callbackObj);

        // the next argument is the element name
    char *cptr = XMLString::transcode(name);
    SV *string = sv_newmortal();
    sv_setpv(string, (char *)cptr);
    XPUSHs(string);

    PUTBACK;

    perl_call_method("end_element", G_VOID);

    FREETMPS;
    LEAVE;
}

void
PerlDocumentCallbackHandler::characters(const XMLCh* const chars, 
				const unsigned int length)
{
    if (!callbackObj) return;

    dSP;

    ENTER;
    SAVETMPS;

    PUSHMARK(SP);
	// first put the callback object on the stack
    XPUSHs(callbackObj);

        // the next argument is the element name
    char *cptr = XMLString::transcode(chars);
    SV *string = sv_newmortal();
    sv_setpv(string, (char *)cptr);
    XPUSHs(string);

        // next is the length
    XPUSHs(sv_2mortal(newSViv(length)));

    PUTBACK;

    perl_call_method("characters", G_VOID);

    FREETMPS;
    LEAVE;
}
void
PerlDocumentCallbackHandler::ignorableWhitespace(const XMLCh* const chars, 
						 const unsigned int length)
{
    if (!callbackObj) return;

    dSP;

    ENTER;
    SAVETMPS;

    PUSHMARK(SP);
	// first put the callback object on the stack
    XPUSHs(callbackObj);

        // the next argument is the element name
    char *cptr = XMLString::transcode(chars);
    SV *string = sv_newmortal();
    sv_setpv(string, (char *)cptr);
    XPUSHs(string);

        // next is the length
    XPUSHs(sv_2mortal(newSViv(length)));

    PUTBACK;

    perl_call_method("ignorable_whitespace", G_VOID);

    FREETMPS;
    LEAVE;
}

void
PerlDocumentCallbackHandler::resetDocument(void)
{
    return;
    if (!callbackObj) return;

    dSP;

    ENTER;
    SAVETMPS;

    PUSHMARK(SP);
	// first put the callback object on the stack
    XPUSHs(callbackObj);

    PUTBACK;

    perl_call_method("reset_document", G_VOID);

    FREETMPS;
    LEAVE;
}

void
PerlDocumentCallbackHandler::startDocument(void)
{
    if (!callbackObj) return;

    dSP;

    ENTER;
    SAVETMPS;

    PUSHMARK(SP);
	// first put the callback object on the stack
    XPUSHs(callbackObj);

    PUTBACK;

    perl_call_method("start_document", G_VOID);

    FREETMPS;
    LEAVE;
}

void
PerlDocumentCallbackHandler::endDocument(void)
{
    if (!callbackObj) return;

    dSP;

    ENTER;
    SAVETMPS;

    PUSHMARK(SP);
	// first put the callback object on the stack
    XPUSHs(callbackObj);

    PUTBACK;

    perl_call_method("end_document", G_VOID);

    FREETMPS;
    LEAVE;
}


void
PerlDocumentCallbackHandler::processingInstruction(const XMLCh* const target,
						   const XMLCh* const data)
{
    if (!callbackObj) return;

    dSP;

    ENTER;
    SAVETMPS;

    PUSHMARK(SP);
	// first put the callback object on the stack
    XPUSHs(callbackObj);

        // the next argument is the target
    char *cptr1 = XMLString::transcode(target);
    SV *string1 = sv_newmortal();
    sv_setpv(string1, (char *)cptr1);
    XPUSHs(string1);

        // the next argument is the data
    char *cptr2 = XMLString::transcode(data);
    SV *string2 = sv_newmortal();
    sv_setpv(string2, (char *)cptr2);
    XPUSHs(string2);

    PUTBACK;

    perl_call_method("processing_instruction", G_VOID);

    FREETMPS;
    LEAVE;
}

void
PerlDocumentCallbackHandler::setDocumentLocator(const Locator* const locator)
{
    if (!callbackObj) return;

    dSP;

    ENTER;
    SAVETMPS;

    PUSHMARK(SP);
	// first put the callback object on the stack
    XPUSHs(callbackObj);

        // next is the attribute list
    char *class_name = "XML::Xerces::Locator";
    HV *hash = newHV();
    HV *stash = gv_stashpv(class_name, FALSE);
    hv_magic(hash, 
    	 (GV *)sv_setref_pv(sv_newmortal(), 
    			    class_name, (void *)locator), 
    	 'P');
    XPUSHs(sv_bless(newRV_noinc((SV *)hash), stash));

    PUTBACK;

    perl_call_method("set_document_locator", G_VOID);

    FREETMPS;
    LEAVE;
}

